#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.GP.Interfaces;

namespace HeuristicLab.GP.StructureIdentification {
  public class VariableFunctionTree : FunctionTreeBase {
    private static readonly IList<IFunctionTree> subTrees = new List<IFunctionTree>().AsReadOnly();
    public double Weight { get; set; }
    public string VariableName { get; set; }
    public int SampleOffset { get; set; }

    public VariableFunctionTree(Variable variable) {
      Function = variable;
    }

    protected VariableFunctionTree(VariableFunctionTree original) {
      Function = original.Function;
      Weight = original.Weight;
      VariableName = original.VariableName;
      SampleOffset = original.SampleOffset;
    }

    public override IList<IFunctionTree> SubTrees {
      get {
        return subTrees;
      }
    }

    public override bool HasLocalParameters {
      get {
        return true;
      }
    }

    public override IOperation CreateShakingOperation(IScope scope) {
      Scope myVariableScope = new Scope();
      scope.AddSubScope(myVariableScope);
      myVariableScope.AddVariable(CreateWeightVariable());
      myVariableScope.AddVariable(CreateVariableNameVariable());
      myVariableScope.AddVariable(CreateSampleOffsetVariable());
      return new AtomicOperation(Function.Manipulator, myVariableScope);
    }

    public override IOperation CreateInitOperation(IScope scope) {
      Scope myVariableScope = new Scope();
      scope.AddSubScope(myVariableScope); 
      myVariableScope.AddVariable(CreateWeightVariable());
      myVariableScope.AddVariable(CreateVariableNameVariable());
      myVariableScope.AddVariable(CreateSampleOffsetVariable());
      return new AtomicOperation(Function.Initializer, myVariableScope);
    }

    public override IFunctionTree Clone() {
      return new VariableFunctionTree(this);
    }

    private IVariable CreateSampleOffsetVariable() {
      IntData data = new IntData(SampleOffset);
      data.Changed += (sender, args) => SampleOffset = data.Data;
      var variable = new HeuristicLab.Core.Variable(Variable.OFFSET, data);
      variable.ValueChanged += (sender, args) => SampleOffset = ((IntData)variable.Value).Data;
      return variable;
    }

    private IVariable CreateVariableNameVariable() {
      StringData data = new StringData(VariableName);
      data.Changed += (sender, args) => VariableName = data.Data;
      var variable = new HeuristicLab.Core.Variable(Variable.VARIABLENAME, data);
      variable.ValueChanged += (sender, args) => VariableName = ((StringData)variable.Value).Data;
      return variable;
    }

    private IVariable CreateWeightVariable() {
      DoubleData data = new DoubleData(Weight);
      data.Changed += (sender, args) => Weight = data.Data;
      var variable = new HeuristicLab.Core.Variable(Variable.WEIGHT, data);
      variable.ValueChanged += (sender, args) => Weight = ((DoubleData)variable.Value).Data;
      return variable;
    }
  }
}
